@@ -28,9 +28,10 @@ from .ops import logical_implication, logical_xor
 if TYPE_CHECKING:
     from mmap import mmap
 
-    PathType = Union[str, PathLike[str]]
     Data = TypeVar("Data", str, bytes)
 
+PathType = Union[str, PathLike]
+
 FILE_IO_BUFFER_SIZE = 8 * 1024 * 1024
 
 
@@ -38,8 +39,7 @@ class Empty(OSError):
     pass
 
 
-def _check_arguments(mode, encoding=None):
-    # type: (str, Optional[str]) -> Optional[str]
+def _check_arguments(mode: str, encoding: Optional[str] = None) -> Optional[str]:
 
     is_text = "t" in mode
     is_binary = "b" in mode
@@ -56,12 +56,13 @@ def _check_arguments(mode, encoding=None):
     return encoding
 
 
-def _stripmode(mode):
+def _stripmode(mode: str) -> str:
     return "".join(set(mode) - {"t", "b"})
 
 
-def read_file(path, mode="b", encoding=None, errors=None):
-    # type: (PathType, str, Optional[str], Optional[str]) -> bytes
+def read_file(
+    path: PathType, mode: str = "b", encoding: Optional[str] = None, errors: Optional[str] = None
+) -> Union[str, bytes]:
 
     """Reads and returns whole file. If content is not needed use consume_file()"""
 
@@ -74,8 +75,13 @@ def read_file(path, mode="b", encoding=None, errors=None):
         return fr.read()
 
 
-def write_file(data, path, mode="wb", encoding=None, errors=None):
-    # type: (Union[str, bytes], PathType, str, Optional[str], Optional[str]) -> None
+def write_file(
+    data: Union[str, bytes],
+    path: PathType,
+    mode: str = "wb",
+    encoding: Optional[str] = None,
+    errors: Optional[str] = None,
+) -> None:
 
     """Writes file."""
 
@@ -114,16 +120,14 @@ def read_or_raise(fin, size):
     return data
 
 
-def get_file_range(path, start, size):
-    # type: (PathType, int, int) -> bytes
+def get_file_range(path: PathType, start: int, size: int) -> bytes:
 
     with open(path, "rb") as fp:
         fp.seek(start)
         return fp.read(size)
 
 
-def truncate_file(path, size):
-    # type: (PathType, int) -> None
+def truncate_file(path: PathType, size: int) -> None:
 
     with open(path, "r+b") as fp:
         fp.truncate(size)
@@ -137,8 +141,17 @@ def wrap_text(bf, mode, encoding, errors, newline):
     return bf
 
 
-def copen(file, mode="rt", archive_file=None, encoding=None, errors=None, newline=None, compresslevel=9, ext=None):
-    # type: (Union[PathType, IO, int], str, Optional[str], Optional[str], Optional[str], Optional[str], int, Optional[str]) -> IO
+def copen(
+    file: Union[PathType, IO, int],
+    mode: str = "rt",
+    archive_file: Optional[str] = None,
+    encoding: Optional[str] = None,
+    errors: Optional[str] = None,
+    newline: Optional[str] = None,
+    compresslevel: int = 9,
+    ext: Optional[str] = None,
+    handle_archives: bool = True,
+) -> IO:
 
     """Generic file open method. It supports transparent compression and improved text-mode handling.
 
@@ -150,6 +163,8 @@ def copen(file, mode="rt", archive_file=None, encoding=None, errors=None, newlin
             Only used if a compressed format is specified.
     `ext`: if `file` is a file-like object, than this can be one of {".gz", ".bz2", ".zip"}
             to enable transparent compression
+    `handle_archives`: Allow transparent handling of archives. Defaults to `True`.
+        If `archive_file` is not given for archives which require it, the archive file will be treated as a normal file.
 
     Returns a file-like object. If `file` was a fd,
             it will be closed when the file-like object is closed.
@@ -157,6 +172,9 @@ def copen(file, mode="rt", archive_file=None, encoding=None, errors=None, newlin
 
     encoding = _check_arguments(mode, encoding)
 
+    if archive_file is not None and handle_archives is False:
+        raise ValueError("handle_archives must be True to allow archive_file to be used")
+
     if isinstance(file, (str, PathLike)):
         ext = os.path.splitext(file)[1].lower()
     elif isinstance(file, int):
@@ -165,30 +183,29 @@ def copen(file, mode="rt", archive_file=None, encoding=None, errors=None, newlin
     else:
         ext = ext and ext.lower()
 
-    if ext == ".gz":
-        import gzip
+    if handle_archives:
+        if ext == ".gz":
+            import gzip
 
-        return gzip.open(file, mode, compresslevel=compresslevel, encoding=encoding, errors=errors, newline=newline)
+            return gzip.open(file, mode, compresslevel=compresslevel, encoding=encoding, errors=errors, newline=newline)
 
-    elif ext == ".bz2":
-        import bz2
+        elif ext == ".bz2":
+            import bz2
 
-        return bz2.open(file, mode, compresslevel=compresslevel, encoding=encoding, errors=errors, newline=newline)
+            return bz2.open(file, mode, compresslevel=compresslevel, encoding=encoding, errors=errors, newline=newline)
 
-    elif ext == ".zip":
-        if archive_file is None:
-            raise ValueError("archive_file must be specified")
+        elif ext == ".zip" and archive_file:
 
-        from zipfile import ZipFile
+            from zipfile import ZipFile
 
-        newmode = _stripmode(mode)
+            newmode = _stripmode(mode)
 
-        with ZipFile(
-            file, newmode
-        ) as zf:  # note: even if the outer zip file is closed, the inner file can still be read apparently
-            bf = zf.open(archive_file, newmode, force_zip64=True)
+            with ZipFile(
+                file, newmode
+            ) as zf:  # note: even if the outer zip file is closed, the inner file can still be read apparently
+                bf = zf.open(archive_file, newmode, force_zip64=True)
 
-        return wrap_text(bf, mode, encoding, errors, newline)
+            return wrap_text(bf, mode, encoding, errors, newline)
 
     if isinstance(file, TextIOWrapper):
         return file
@@ -266,6 +283,8 @@ class StdoutFile:
     def __init__(self, path=None, mode="xb", encoding=None, errors=None, newline=None, compresslevel=9):
         # type: (Optional[PathType], str, Optional[str], Optional[str], Optional[str], int) -> None
 
+        encoding = _check_arguments(mode, encoding)
+
         if path:
             self.fp = copen(path, mode, encoding=encoding, errors=errors, newline=newline, compresslevel=compresslevel)
             self.doclose = True
@@ -750,7 +769,7 @@ def iter_zip(file, mode="rb", encoding=None, errors=None, newline=None, password
 
 
 def iter_7zip(
-    file: Union[BinaryIO, str, PathLike],
+    file: Union[BinaryIO, PathType],
     mode: str = "rb",
     encoding: Optional[str] = None,
     errors: Optional[str] = None,
@@ -799,7 +818,7 @@ def iter_tar(file, mode="rb", encoding=None, errors=None, newline=None):
 
 
 def iter_dir(
-    path: str,
+    path: PathType,
     mode: str = "rb",
     encoding: Optional[str] = None,
     errors: Optional[str] = None,
@@ -847,8 +866,13 @@ def iter_dir(
                 yield from iter_dir(entry.path, mode, encoding, errors, newline, follow_symlinks, archives)
 
 
-def iter_lines(path, encoding="utf-8", errors="strict", newline=None, verbose=False):
-    # type: (str, str, str, Optional[str], bool) -> Iterator[str]
+def iter_lines(
+    path: PathType,
+    encoding: str = "utf-8",
+    errors: str = "strict",
+    newline: Optional[str] = None,
+    verbose: bool = False,
+) -> Iterator[str]:
 
     """Yield lines from text file. Handles compressed files as well.
     if verbose is True, print a progress bar
@@ -879,8 +903,9 @@ def iter_lines(path, encoding="utf-8", errors="strict", newline=None, verbose=Fa
                 yield line
 
 
-def iter_stripped(path, encoding="utf-8", errors="strict", newline=None):
-    # type: (str, str, str, Optional[str]) -> Iterator[str]
+def iter_stripped(
+    path: PathType, encoding: str = "utf-8", errors: str = "strict", newline: Optional[str] = None
+) -> Iterator[str]:
 
     """Yield lines from text file with trailing newlines stripped.
     Handles compressed files as well.
