@@ -1,14 +1,11 @@
 from __future__ import generator_stop
 
-from typing import TYPE_CHECKING
+from typing import Any, Callable, Dict, Iterable, Mapping, Optional
 
 from werkzeug.exceptions import BadRequest
 from werkzeug.wsgi import get_host
 
-if TYPE_CHECKING:
-    from typing import Any, Callable, Dict, Iterable, Mapping, Optional
-
-    WsgiApp = Any
+WsgiApp = Any
 
 
 class HostDispatcher:
@@ -23,8 +20,7 @@ class HostDispatcher:
         self.hosts = hosts
         self.default = default or BadRequest()  # rfc7230#section-5.4
 
-    def __call__(self, environ, start_response):
-        # type: (Mapping, Callable) -> Iterable[bytes]
+    def __call__(self, environ: Mapping[str, Any], start_response: Callable) -> Iterable[bytes]:
 
         host = get_host(environ)
         app = self.hosts.get(host, self.default)
